package org.sadigit.entity.transaksi;

import java.io.Serializable;
import java.time.LocalDateTime;

import org.hibernate.annotations.Immutable;
import org.locationtech.jts.geom.Geometry;
import org.sadigit.adapter.GeometryAdapter;
import org.sadigit.entity.referensi.MasterKelurahan;
import org.sadigit.entity.referensi.Ulp;

import io.quarkus.hibernate.orm.panache.PanacheEntityBase;
import jakarta.json.bind.annotation.JsonbTypeAdapter;
import jakarta.persistence.*;
import lombok.Getter;
import lombok.Setter;

@Entity
@Setter
@Getter
@Table(name = "keluhan", schema = "apkt_transactional")
@Immutable
public class Keluhan extends PanacheEntityBase implements Serializable {
    @Id
    @SequenceGenerator(name = "keluhan_seq", sequenceName = "apkt_transactional.keluhan_seq", allocationSize = 1)
    @GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "keluhan_seq")
    @Column(name = "id")
    private Long id;

    @Column(name = "id_pelanggan_no_meter")
    private Long id_pelanggan_no_meter;

    @ManyToOne(fetch = FetchType.EAGER)
    @JoinColumn(name = "id_ulp", foreignKey = @ForeignKey(name = "id_ulp"))
    private Ulp ulp;
    @Column(name = "id_tipe_permasalahan")
    private Long id_tipe_permasalahan;

    @Column(name = "alamat_pelanggan")
    private String alamat_pelanggan;

    @Column(name = "daya_tarif")
    private String daya_tarif;

    @Column(name = "nama_pelapor")
    private String nama_pelapor;

    @Column(name = "telepon_pelapor")
    private String telepon_pelapor;

    @Column(name = "hp_pelapor")
    private String hp_pelapor;

    @Column(name = "fax_pelapor")
    private String fax_pelapor;

    @Column(name = "email_pelapor")
    private String email_pelapor;

    @Column(name = "alamat_pelapor")
    private String alamat_pelapor;

    @Column(name = "media")
    private String media;

    @Column(name = "waktu_padam")
    private String waktu_padam;

    @Column(name = "konfirmasi")
    private String konfirmasi;

    @JsonbTypeAdapter(GeometryAdapter.class)
    @Column(name = "lokasi", columnDefinition = "geometry(Point,4326)")
    private Geometry lokasi;

    @Column(name = "no_laporan", unique = true)
    private String no_laporan;

    @Column(name = "keterangan")
    private String keterangan;

    @Column(name = "created_date")
    private LocalDateTime created_date;

    @Column(name = "created_by")
    private Long created_by;

    @Column(name = "updated_date")
    private LocalDateTime updated_date;

    @Column(name = "updated_by")
    private Long updated_by;

    @Column(name = "status_akhir")
    private String status_akhir;

    @Column(name = "tipe_wo")
    private Long tipe_wo;

    @Column(name = "unit")
    private Long unit;

    @Column(name = "eskalasi")
    private Long eskalasi;

    @Column(name = "permasalahan")
    private String permasalahan;

    @Column(name = "waktu_lapor")
    private LocalDateTime waktu_lapor;

    @Column(name = "waktu_penugasan")
    private LocalDateTime waktu_penugasan;

    @Column(name = "waktu_perjalanan")
    private LocalDateTime waktu_perjalanan;

    @Column(name = "waktu_pengerjaan")
    private LocalDateTime waktu_pengerjaan;

    @Column(name = "waktu_nyala_sementara")
    private LocalDateTime waktu_nyala_sementara;

    @Column(name = "waktu_nyala")
    private LocalDateTime waktu_nyala;

    @Column(name = "waktu_batal")
    private LocalDateTime waktu_batal;

    @Column(name = "waktu_selesai")
    private LocalDateTime waktu_selesai;

    @Column(name = "tipe_keluhan")
    private Long tipe_keluhan;

    @Column(name = "tipe_sub_keluhan")
    private Long tipe_sub_keluhan;

    @Column(name = "alasan")
    private String alasan;

    @Column(name = "patokan")
    private String patokan;

    @Column(name = "jarak")
    private Long jarak;
    
    @ManyToOne(fetch = FetchType.EAGER)
    @JoinColumn(name = "id_kelurahan", referencedColumnName = "id_kelurahan", foreignKey = @ForeignKey(name = "kelurahan"))
    private MasterKelurahan kelurahan;

    @ManyToOne(fetch = FetchType.EAGER)
    @JoinColumn(name = "id_regu", referencedColumnName = "id" )
    private MasterRegu regu;

    @Column(name = "fax")
    private String fax;
    @Column(name = "keterangan_lokasi")
    private String keterangan_lokasi;
    @Column(name = "waktu_fax")
    private LocalDateTime waktu_fax;
    @Column(name = "nama_media")
    private String nama_media;
    @Column(name = "waktu_media")
    private LocalDateTime waktu_media;
}
