package org.sadigit.model.request;

import java.util.Map;

import org.sadigit.control.exception.CustomException;
import org.sadigit.model.response.base.ResponseModel;

import jakarta.validation.constraints.NotNull;
import jakarta.ws.rs.QueryParam;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class CreateGangguanV2Request {

    @QueryParam("in_idpel")
    private String in_idpel;

    @NotNull
    @QueryParam("in_nama")
    private String in_nama;

    @NotNull
    @QueryParam("in_alamat")
    private String in_alamat;

    @NotNull
    @QueryParam("in_hp")
    private String in_hp;

    @NotNull
    @QueryParam("in_unitup")
    private String in_unitup;

    @NotNull
    @QueryParam("in_long")
    private String in_long;

    @NotNull
    @QueryParam("in_lat")
    private String in_lat;

    @NotNull
    @QueryParam("in_tipe_keluhan")
    private String in_tipe_keluhan;

    @NotNull
    @QueryParam("in_keterangan")
    private String in_keterangan;

    @NotNull
    @QueryParam("in_prov")
    private String in_prov;

    @NotNull
    @QueryParam("in_kab")
    private String in_kab;

    @NotNull
    @QueryParam("in_kec")
    private String in_kec;

    @NotNull
    @QueryParam("in_kel")
    private String in_kel;

    @QueryParam("in_flag_padam_tm")
    private String in_flag_padam_tm;

    @QueryParam("in_laporan_ref")
    private String in_laporan_ref;

    @QueryParam("in_kode_gardu")
    private String in_kode_gardu;

    @QueryParam("in_tgl_konfirmasi")
    private String in_tgl_konfirmasi;

    @QueryParam("in_laporanulang_ref")
    private String in_laporanulang_ref;

    public void checkParameter() {
        if (this.getIn_kel() == null || this.getIn_kel().isEmpty()) {
            var error = ResponseModel.builder()
                    .data(Map.of(
                            "msg", "nama kelurahan tidak boleh kosong",
                            "result", "0"))
                    .build();
            throw new CustomException(null, error);
        }

        if (this.getIn_kec() == null || this.getIn_kec().isEmpty()) {
            var error = ResponseModel.builder()
                    .data(Map.of(
                            "msg", "Nama kecamatan tidak boleh kosong",
                            "result", "0"))
                    .build();
            throw new CustomException(null, error);
        }

        if (this.getIn_kab() == null || this.getIn_kab().isEmpty()) {
            var error = ResponseModel.builder()
                    .data(Map.of(
                            "msg", "Nama kabupaten tidak boleh kosong",
                            "result", "0"))
                    .build();
            throw new CustomException(null, error);
        }

        if (this.getIn_prov() == null || this.getIn_prov().isEmpty()) {
            var error = ResponseModel.builder()
                    .data(Map.of(
                            "msg", "Nama provinsi tidak boleh kosong",
                            "result", "0"))
                    .build();
            throw new CustomException(null, error);
        }

        if (this.getIn_nama() == null || this.getIn_nama().isEmpty()) {
            var error = ResponseModel.builder()
                    .data(Map.of(
                            "msg", "Nama tidak boleh kosong",
                            "result", "0"))
                    .build();
            throw new CustomException(null, error);
        }

        if (this.getIn_alamat() == null || this.getIn_alamat().isEmpty()) {
            var error = ResponseModel.builder()
                    .data(Map.of(
                            "msg", "Alamat tidak boleh kosong",
                            "result", "0"))
                    .build();
            throw new CustomException(null, error);
        }

        if (this.getIn_hp() == null || this.getIn_hp().isEmpty()) {
            var error = ResponseModel.builder()
                    .data(Map.of(
                            "msg", "No telp tidak boleh kosong",
                            "result", "0"))
                    .build();
            throw new CustomException(null, error);
        }

        if (this.getIn_unitup() == null || this.getIn_unitup().isEmpty()) {
            var error = ResponseModel.builder()
                    .data(Map.of(
                            "msg", "Unitup tidak boleh kosong",
                            "result", "0"))
                    .build();
            throw new CustomException(null, error);
        }

        if (this.getIn_long() == null || this.getIn_long().isEmpty()) {
            var error = ResponseModel.builder()
                    .data(Map.of(
                            "msg", "Longitude tidak boleh kosong",
                            "result", "0"))
                    .build();
            throw new CustomException(null, error);
        }

        if (this.getIn_lat() == null || this.getIn_lat().isEmpty()) {
            var error = ResponseModel.builder()
                    .data(Map.of(
                            "msg", "Latitude tidak boleh kosong",
                            "result", "0"))
                    .build();
            throw new CustomException(null, error);
        }

        if (this.getIn_tipe_keluhan() == null || this.getIn_tipe_keluhan().isEmpty()) {
            var error = ResponseModel.builder()
                    .data(Map.of(
                            "msg", "Tipe keluhan tidak boleh kosong",
                            "result", "0"))
                    .build();
            throw new CustomException(null, error);
        }

        if (this.getIn_keterangan() == null || this.getIn_keterangan().isEmpty()) {
            var error = ResponseModel.builder()
                    .data(Map.of(
                            "msg", "Keterangan tidak boleh kosong",
                            "result", "0"))
                    .build();
            throw new CustomException(null, error);
        }
    }

    public static CreateGangguanV2Request fromCreateGangguanV1(CreateGangguanRequest request) {
        return CreateGangguanV2Request.builder()
                .in_idpel(request.getIn_idpel())
                .in_nama(request.getIn_nama())
                .in_alamat(request.getIn_alamat())
                .in_hp(request.getIn_hp())
                .in_unitup(request.getIn_unitup())
                .in_long(request.getIn_long())
                .in_lat(request.getIn_lat())
                .in_tipe_keluhan(request.getIn_tipe_keluhan())
                .in_keterangan(request.getIn_keterangan())
                .in_prov(request.getIn_prov())
                .in_kab(request.getIn_kab())
                .in_kec(request.getIn_kec())
                .in_kel(request.getIn_kel())
                .in_flag_padam_tm(request.getIn_flag_padam_tm())
                .in_laporan_ref(request.getIn_laporan_ref())
                .build();
    }

}
