package org.sadigit.repository.entity.transaksi;

import jakarta.enterprise.context.ApplicationScoped;
import jakarta.inject.Inject;
import jakarta.transaction.Transactional;

import org.locationtech.jts.geom.Point;
import org.locationtech.jts.io.ParseException;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.Month;
import java.time.format.DateTimeFormatter;
import java.util.Map;

import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.sadigit.control.exception.CustomException;
import org.sadigit.model.request.CreateGangguanRequest;
import org.sadigit.model.request.CreateGangguanV2Request;
import org.sadigit.model.response.CreateGangguanResponse;
import org.sadigit.model.response.base.ResponseModel;
import org.sadigit.entity.masterYantek.Unit;
import org.sadigit.entity.referensi.MasterKelurahan;
import org.sadigit.entity.referensi.Media;
import org.sadigit.entity.referensi.TipePermasalahan;
import org.sadigit.entity.referensi.Uid;
import org.sadigit.entity.transaksi.Gangguan;
import org.sadigit.entity.transaksi.GangguanLaporUlang;
import org.sadigit.entity.transaksi.GangguanTransaksi;
import org.sadigit.utils.GeomUtils;

import io.quarkus.hibernate.orm.panache.PanacheRepository;

@ApplicationScoped
public class GangguanRepository implements PanacheRepository<Gangguan> {

    @Inject
    Session session;

    private String generateNoLaporan(Long id_uid) {
        // select unitcode from unit
        Unit unit = Unit.find("unitid", id_uid).firstResult();
        if (unit == null) {
            throw new CustomException("Unit tidak ditemukan");
        }
        String unitCode = unit.getUnitcode();
        String noLaporan = "";
        String prefix = "G";
        Gangguan gangguan = Gangguan.find("unit =?1 and date(created_date)=?2 order by created_date desc", id_uid, LocalDate.now())
                .firstResult();
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyMMdd");
        if (gangguan != null) {
            String lastNoLaporan = gangguan.getNo_laporan();
            String currentDate = LocalDate.now().format(formatter);
            String lastNoLaporanNumber = lastNoLaporan.substring(lastNoLaporan.length() - 4);
            int lastNoLaporanNumberInt = Integer.parseInt(lastNoLaporanNumber);
            int newNoLaporanNumber = lastNoLaporanNumberInt + 1;
            String newNoLaporanNumberString = String.format("%05d", newNoLaporanNumber);
            noLaporan = prefix + unitCode + currentDate + newNoLaporanNumberString;
            System.out.println("Generated no laporan: " + noLaporan);

            return noLaporan;
        } else {
            LocalDate date = LocalDate.now(); // Get current date directly
            System.out.println("Parsed date: " + date);
            // Format the LocalDate object to a string using the formatter
            String formattedDate = date.format(formatter);
            System.out.println("Formatted date: " + formattedDate);
            noLaporan = prefix + unitCode + formattedDate + "0001";
            System.out.println("Generated no laporan: " + noLaporan);
            return noLaporan;
        }
    }

    @Transactional
    public CreateGangguanResponse create(CreateGangguanRequest prmt) {

        if (prmt.getIn_idpel() != null) {
            LocalDateTime now = LocalDateTime.now();
            LocalDateTime earlier = now.minusHours(3);
            Gangguan gangguanExist = Gangguan.find("id_pelanggan_no_meter = ?1 and created_date between ?2 and ?3", prmt.getIn_idpel(), earlier, now).firstResult();
            if (gangguanExist != null) {
                var error = ResponseModel.builder()
                        .data(Map.of("msg", "Pengaduan anda sedang di proses. Anda dapat melakukan pengaduan ulang, 3 jam dari pengaduan sebelumnya."))
                        .build();
                throw new CustomException(null, error);
            }
        }

        try {

            int sourceSrid = 4326;
            Gangguan gangguan = new Gangguan();
            Media media = Media.findById(10);
            GangguanTransaksi gangguanTransaksi = new GangguanTransaksi();
            String noLaporan = "";
            Point point = null;

            try {
                point = GeomUtils.wktToPoint(prmt.getIn_long(), prmt.getIn_lat(), sourceSrid);
            } catch (ParseException e) {
                throw new CustomException("Invalid WKT format: " + e.getMessage());
            }
            
            MasterKelurahan kelurahan = MasterKelurahan.find("nama_kelurahan", prmt.getIn_kel()).firstResult();
            TipePermasalahan tipePermasalahan = TipePermasalahan.findById(Long.valueOf(prmt.getIn_tipe_keluhan()));
            Uid uid = Uid.findById(prmt.getIn_unitup());

            if (uid == null) {
                throw new CustomException("Unit tidak ditemukan");
            }

            if (tipePermasalahan == null) {
                throw new CustomException("Tipe Permasalahan tidak ditemukan");
            }

            if (prmt.getIn_kel() != null && kelurahan == null) {
                throw new CustomException("Kelurahan tidak ditemukan");
            }
            if (prmt.getIn_idpel() != null) {
                gangguan.setId_pelanggan_no_meter(Long.valueOf(prmt.getIn_idpel()));
            }

            gangguan.setTipePermasalahan(tipePermasalahan);
            gangguan.setMaster_kelurahan(kelurahan);
            noLaporan = generateNoLaporan(Long.valueOf(prmt.getIn_unitup()));
            gangguan.setNo_laporan(noLaporan);
            gangguan.setNama_pelapor(prmt.getIn_nama());
            gangguan.setTelepon_pelapor(prmt.getIn_hp());
            gangguan.setHp_pelapor(prmt.getIn_hp());
            gangguan.setAlamat_pelapor(prmt.getIn_alamat());

            gangguan.setMedia(media.getKeterangan());
            gangguan.setWaktu_lapor(LocalDateTime.now());

            gangguan.setLokasi(point);
            gangguan.setKeterangan(prmt.getIn_keterangan());
            gangguan.setCreated_date(LocalDateTime.now());
            gangguan.setCreated_by(99999L);
            gangguan.setStatus_akhir("lapor");
            gangguan.setUnit(Long.valueOf(prmt.getIn_unitup()));
            gangguan.setPermasalahan(tipePermasalahan.getNama());
            gangguan.setCreated_date(LocalDateTime.now());
            gangguan.persist();

            gangguanTransaksi.setGangguan(gangguan);
            gangguanTransaksi.setJenis("lapor");
            gangguanTransaksi.setWaktu(LocalDateTime.now());
            gangguanTransaksi.setCreated_date(LocalDateTime.now());
            gangguanTransaksi.setCreated_by(99999L);
            GangguanTransaksi.persist(gangguanTransaksi);

            if (!isPersistent(gangguan)) {
                Gangguan.delete("no_laporan", gangguan.getNo_laporan());
                throw new CustomException("Gangguan tidak berhasil disimpan");
            }

            return new CreateGangguanResponse(noLaporan);
        } catch (HibernateException e) {
            throw new CustomException("Terjadi kesalahan dalam proses menyimpan data : " + e.getMessage());
        }
    }

    public CreateGangguanResponse createV2(CreateGangguanV2Request prmt) {

        if (prmt.getIn_idpel() != null) {
            LocalDateTime now = LocalDateTime.now();
            LocalDateTime earlier = now.minusHours(3);
            Gangguan gangguanExist = Gangguan.find("id_pelanggan_no_meter = ?1 and created_date between ?2 and ?3", prmt.getIn_idpel(), earlier, now).firstResult();
            if (gangguanExist != null) {
                var error = ResponseModel.builder()
                    .data(Map.of(
                        "msg", "Pengaduan anda sedang di proses. Anda dapat melakukan pengaduan ulang, 3 jam dari pengaduan sebelumnya.",
                        "result", "0"
                    ))
                    .build();
                throw new CustomException(null, error);
            }
        }

        try {
            int sourceSrid = 4326;
            Gangguan gangguan = new Gangguan();
            Media media = Media.findById(10);
            GangguanTransaksi gangguanTransaksi = new GangguanTransaksi();
            String noLaporan = "";
    
            try {
                Point point = GeomUtils.wktToPoint(prmt.getIn_long(), prmt.getIn_lat(), sourceSrid);
                MasterKelurahan kelurahan = MasterKelurahan.find("nama_kelurahan", prmt.getIn_kel()).firstResult();
                TipePermasalahan tipePermasalahan = TipePermasalahan.findById(Long.valueOf(prmt.getIn_tipe_keluhan()));
                noLaporan = generateNoLaporan(Long.valueOf(prmt.getIn_unitup()));
                Uid uid = Uid.findById(prmt.getIn_unitup());
                if (uid == null) {
                    throw new CustomException("Unit tidak ditemukan");
                }

                if (tipePermasalahan == null) {
                    throw new CustomException("Tipe Permasalahan tidak ditemukan");
                }

                if (kelurahan == null) {
                    throw new CustomException("Kelurahan tidak ditemukan");
                }
                if (prmt.getIn_idpel() != null) {
                    gangguan.setId_pelanggan_no_meter(Long.valueOf(prmt.getIn_idpel()));
                }

                gangguan.setTipePermasalahan(tipePermasalahan);
                gangguan.setMaster_kelurahan(kelurahan);

                if (prmt.getIn_laporanulang_ref() != null) {
                    Gangguan gangguanExist = Gangguan.find("no_laporan = ?1", prmt.getIn_laporanulang_ref()).firstResult();
                    if (gangguanExist == null) {
                        var error = ResponseModel.builder()
                            .data(Map.of(
                                "msg", "Referensi no laporan tidak ditemukan",
                                "result", "0"
                            ))
                            .build();
                        throw new CustomException(null, error);
                    }
        
                    Long gangguanLaporUlang = GangguanLaporUlang.find("no_laporan_referensi", prmt.getIn_laporanulang_ref()).count();
                    GangguanLaporUlang newGangguanLaporUlang = new GangguanLaporUlang();
                    newGangguanLaporUlang.setLaporan_ke(gangguanLaporUlang);
                    newGangguanLaporUlang.setNo_laporan_referensi(prmt.getIn_laporanulang_ref());
                    newGangguanLaporUlang.setNo_laporan_baru(noLaporan);
                    newGangguanLaporUlang.setGangguan(gangguanExist);
                    GangguanLaporUlang.persist(newGangguanLaporUlang);
                }

                if (prmt.getIn_tgl_konfirmasi() != null) {

                    if (prmt.getIn_tgl_konfirmasi().length() < 12) {
                        var error = ResponseModel.builder()
                        .data(Map.of(
                            "msg", "Panjang tgl Konfirmasi kurang dari 12 karakter",
                            "result", "0"
                        )).build();
                        throw new CustomException(null, error);
                    }

                    int tahun = Integer.valueOf(prmt.getIn_tgl_konfirmasi().substring(0,4));
                    int bulan = Integer.valueOf(prmt.getIn_tgl_konfirmasi().substring(4,6));
                    int tanggal = Integer.valueOf(prmt.getIn_tgl_konfirmasi().substring(6,8));
                    int jam = Integer.valueOf(prmt.getIn_tgl_konfirmasi().substring(8, 10));
                    int menit = Integer.valueOf(prmt.getIn_tgl_konfirmasi().substring(10, 12));

                    LocalDateTime tglKonfirmasi = LocalDateTime.of(tahun, Month.of(bulan), tanggal, jam, menit);
                    gangguan.setWaktu_konfirmasi(tglKonfirmasi);
                }
                
                gangguan.setNo_laporan(noLaporan);
                gangguan.setNama_pelapor(prmt.getIn_nama());
                gangguan.setTelepon_pelapor(prmt.getIn_hp());
                gangguan.setHp_pelapor(prmt.getIn_hp());
                gangguan.setAlamat_pelapor(prmt.getIn_alamat());

                gangguan.setMedia(media.getKeterangan());
                gangguan.setWaktu_lapor(LocalDateTime.now());

                gangguan.setLokasi(point);
                gangguan.setKeterangan(prmt.getIn_keterangan());
                gangguan.setCreated_date(LocalDateTime.now());
                gangguan.setCreated_by(99999L);
                gangguan.setStatus_akhir("LAPOR");
                gangguan.setUnit(Long.valueOf(prmt.getIn_unitup()));
                gangguan.setPermasalahan(tipePermasalahan.getNama());
                gangguan.setCreated_date(LocalDateTime.now());
                gangguan.persist();

                gangguanTransaksi.setGangguan(gangguan);
                gangguanTransaksi.setJenis("LAPOR");
                gangguanTransaksi.setWaktu(LocalDateTime.now());
                gangguanTransaksi.setCreated_date(LocalDateTime.now());
                gangguanTransaksi.setCreated_by(99999L);

                GangguanTransaksi.persist(gangguanTransaksi);

                if (!isPersistent(gangguan)) {
                    Gangguan.delete("no_laporan", gangguan.getNo_laporan());
                    throw new CustomException("Gangguan tidak berhasil disimpan");
                }

            } catch (ParseException e) {
                System.err.println("Invalid WKT format: " + e.getMessage());
                throw new CustomException("Invalid WKT format: " + e.getMessage());
            }

            return new CreateGangguanResponse(noLaporan);
        } catch (HibernateException e) {
            throw new CustomException("Terjadi kesalahan dalam proses menyimpan data : " + e.getMessage());
        }
    }

}
